import pytest
from downerhelper.logs import PostgresLogQueue, get_config_dict
from downerhelper.secrets import get_secret_value
import os
from uuid import uuid4
import psycopg2 as pg

def test_class_valid_queue():
    db_config_str = get_secret_value('db-config-1', os.getenv('KEYVAULT_URL'))
    db_config_split = db_config_str.split(',')
    keys = ['dbname', 'user', 'password', 'host']
    db_config = {k: v for k, v in zip(keys, db_config_split)}

    get_config_dict('db-config-1', os.getenv('KEYVAULT_URL'))

    logger_name = str(uuid4())
    job_id = str(uuid4())
    table = 'pytest_logs_queue'

    queue = PostgresLogQueue(logger_name, job_id, db_config, table)

    messages = [str(uuid4()) for _ in range(4)]
    for message in messages:
        queue.add('INFO', message)

    queue.save()

    try:
        conn = pg.connect(**db_config)
        cur = conn.cursor()
        cur.execute(f"""select name, job_id 
        from {table}
        where job_id = '{job_id}';""")
        rows = cur.fetchall()
    except:
        assert False

    assert len(rows) >= len(messages)
    for row in rows:
        assert row[0] == logger_name
        assert row[1] == job_id

def test_invalid_queue():
    db_config_str = get_secret_value('db-config-1', os.getenv('KEYVAULT_URL'))
    db_config_split = db_config_str.split(',')
    keys = ['dbname', 'user', 'password', 'host']
    db_config = {k: v for k, v in zip(keys, db_config_split)}

    with pytest.raises(Exception):
        PostgresLogQueue(None, 'a', 'a', db_config)

    with pytest.raises(Exception):
        PostgresLogQueue('a', None, 'a', db_config)

    with pytest.raises(Exception):
        PostgresLogQueue('a', 'a', None, db_config)

    with pytest.raises(Exception):
        PostgresLogQueue('a', 'a', 'a', {})


from . import PostgresLogHandler
from ..secrets import get_secret_value
import logging 
from uuid import uuid4

def setup_handler(secret_name, keyvault_url, logger_name, table, job_id=None):
    try:
        db_config = get_config_dict(secret_name, keyvault_url)
        if not job_id:
            job_id = str(uuid4())
            logging.warning(f"Job ID: {job_id}")
        logger = PostgresLogHandler(logger_name, job_id, table, db_config)
        return logger, job_id
    except Exception as e:
        logging.error(f"Error setting up handler: {e}")
        return None, None
    
def get_config_dict(secret_name, keyvault_url):
    db_config_str = get_secret_value(secret_name, keyvault_url)
    db_config_split = db_config_str.split(',')
    keys = ['dbname', 'user', 'password', 'host']
    return {k: v for k, v in zip(keys, db_config_split)}



import pytest
from downerhelper.secrets import get_secret_value
import os 
import logging 

def test_valid_secret():
    value = get_secret_value('test-secret', os.getenv('KEYVAULT_URL'))
    assert value == 'dfaec67d-cf0a-44e8-b5df-6e8346092450'

def test_valid_secret_logging():
    value = get_secret_value('test-secret', os.getenv('KEYVAULT_URL'), logger=logging.getLogger('test'))
    assert value == 'dfaec67d-cf0a-44e8-b5df-6e8346092450'

def test_invalid_secret():
    value = get_secret_value('invalid', os.getenv('KEYVAULT_URL'))
    assert value == None

def test_invalid_keyvault():
    value = get_secret_value('test-secret', 'https://invalid.vault.azure.net')
    assert value == None




import logging
import psycopg2 as pg

class PostgresLogHandler(logging.Handler):
    def __new__(cls, logger_name, job_id, table, db_config):
        try:
            instance = super(PostgresLogHandler, cls).__new__(cls)
            instance.__init__(logger_name, job_id, table, db_config)
            return instance.logger
        except Exception as e:
            logging.error(f"Error creating PostgresLogHandler: {e}")
            return None

    def __init__(self, logger_name, job_id, table, db_config):
        try:
            if '' in [logger_name, job_id, table] or None in [logger_name, job_id, table]:
                raise Exception("Invalid parameters")
            logging.Handler.__init__(self)
            self.connection = pg.connect(**db_config)
            self.cursor = self.connection.cursor()
            self.logger_name = logger_name
            self.table = table
            self.job_id = job_id
            self._create_table_if_not_exists()
            self.logger = self._setup_logging()
        except Exception as e:
            logging.error(f"Error setting up PostgresLogHandler: {e}")

    def _create_table_if_not_exists(self):
        try:
            self.cursor.execute("set time zone 'UTC'")
            self.cursor.execute(f"""
            create table if not exists {self.table} (
                id serial primary key,
                created_at timestamptz default now(),
                name varchar(255),
                levelname varchar(50),
                message text,
                job_id varchar(255) not null
            )""", (self.table,))
            self.connection.commit()
        except Exception as e:
            logging.error(f"Error creating table {self.table}: {e}")

    def _setup_logging(self):
        try:
            logger = logging.getLogger(self.logger_name)
            logger.setLevel(logging.DEBUG)

            self.setLevel(logging.DEBUG)

            formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
            self.setFormatter(formatter)

            logger.addHandler(self)
            logger.info(f"Logger: {self.logger_name} setup with job_id: {self.job_id}")

            return logger
        except Exception as e:
            logging.error(f"Error setting up logging: {e}")        

    def emit(self, record):
        try:
            log_entry = self.format(record)
            self.cursor.execute(f"""
            insert into {self.table}
            (name, levelname, message, job_id) 
            values 
            (%s, %s, %s, %s)
            """, (record.name, record.levelname, log_entry, self.job_id))
            self.connection.commit()
        except Exception as e:
            logging.error(f"Error emitting log record: {e}")

    def close(self):
        try:
            self.cursor.close()
            self.connection.close()
            logging.Handler.close(self)
        except Exception as e:
            logging.error(f"Error closing connection: {e}")


import pytest
from downerhelper.logs import setup_handler, get_config_dict
from downerhelper.secrets import get_secret_value
import os
from uuid import uuid4
import psycopg2 as pg

def test_get_config_dict():
    secret_name = 'db-config-test'
    db_config = get_config_dict(secret_name, os.getenv('KEYVAULT_URL'))
    assert db_config == {
        'dbname': 'value0',
        'user': 'value1',
        'password': 'VALue2',
        'host': 'value3'
    }

    with pytest.raises(Exception):
        get_config_dict('invalid', os.getenv('KEYVAULT_URL'))

def test_setup_handler():
    secret_name = 'db-config-1'
    table = os.getenv('LOG_TABLE')
    logger_name = str(uuid4())
    logger, job_id = setup_handler(secret_name, os.getenv('KEYVAULT_URL'), logger_name, table)
    assert logger != None
    assert isinstance(job_id, str)

    messages = [str(uuid4()) for _ in range(4)]
    for message in messages:
        logger.info(message)

    db_config_str = get_secret_value(secret_name, os.getenv('KEYVAULT_URL'))
    db_config_split = db_config_str.split(',')
    keys = ['dbname', 'user', 'password', 'host']
    db_config = {k: v for k, v in zip(keys, db_config_split)}

    try:
        conn = pg.connect(**db_config)
        cur = conn.cursor()
        cur.execute(f"""select name, job_id 
        from {table}
        where job_id = '{job_id}';""")
        rows = cur.fetchall()
    except:
        assert False

    assert len(rows) >= len(messages)
    for row in rows:
        assert row[0] == logger_name
        assert row[1] == job_id

    set_job_id = str(uuid4())
    logger, job_id = setup_handler(secret_name, os.getenv('KEYVAULT_URL'), logger_name, table, job_id=set_job_id)
    assert logger != None
    assert job_id == set_job_id
    
def test_setup_handler_invalid():
    table = os.getenv('LOG_TABLE')
    logger_name = str(uuid4())
    logger, job_id = setup_handler('invalid', os.getenv('KEYVAULT_URL'), logger_name, table)
    assert logger == job_id == None


from azure.keyvault.secrets import SecretClient
from azure.identity import DefaultAzureCredential
import logging

def get_secret_value(secret_name, vault_url, credential=DefaultAzureCredential(), logger=None):
    try:
        if not logger:
            logger = logging.getLogger(__name__)
        client = SecretClient(vault_url=vault_url, credential=credential)
        secret = client.get_secret(secret_name)
        return secret.value
    except Exception as e:
        logger.error(f"Error getting secret {secret_name}: {e}")
    return None